/* *******************************************************************************************
The MIT License

Copyright (c) 2007 Steven G. Greenberg

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons
to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
******************************************************************************************* */

#pragma once

#include "AsfFile.h"

// and here's the max length of the extension info we'll store
#define MAX_PAYLOAD_EXT_SYS_BYTES	256

struct PAYLOAD_EXTENSION_SYSTEMS
{
	GUID	guid;
	WORD	wDataSize;
	DWORD	dwExtInfoLen;
	BYTE	uchExtInfo[MAX_PAYLOAD_EXT_SYS_BYTES];
};

#define MAX_PAYLOAD_EXTENSION_SYSTEMS	8

class CAsfStream
{
public:

	CAsfStream(CFileCache2* pOurInputFileCache, DWORD dwStreamNum, DWORD dwPacketSize);

	BOOL		AsfDataDemux(HANDLE hOutputFile);

	DWORD		GetAsfFramePeriod(void);
	void		GetAsfPar(DWORD* pdwHorizSize, DWORD* pdwVertSize);

	void		SetSeqAndEntryHdr(const struct VC1_HEADER_SEQUENCE* pSeq);
	void		SeqEntryHdrOnly(const struct VC1_HEADER_SEQUENCE* pSeq);
	void		GetAsfVc1Hdr(struct VC1_HEADER_SEQUENCE* pSeq);

// called by our parent class only:
	DWORD		ProcMetaDataDescripRec(struct ASF_METADATA_DESCRIPTION_RECORD* pMetaDescripRec, DWORD dwBytesLeft);
	BOOL		ProcStreamPropObj(struct STREAM_PROPERTIES_OBJECT* pStreamPropObj, BOOL bExtraIndent);
	BOOL		ProcASF_Extended_Stream_Properties_Object(struct EXTENDED_STREAM_PROPERTIES_OBJECT* pExtStreamObj);

	const char* StreamNameOfGuid(GUID guid);

protected:
	CFileCache2*	m_pInputFileCache;
	DWORD			m_dwPacketSize;
	DWORD			m_dwOurStreamNum;
	BOOL			m_bIsVideo;
	BOOL			m_bIsVc1;
	BOOL			m_bIsInterlaced;	// this is an "input", so to speak, it should be set
	BOOL			m_bUseEntryHdrOnly;	// this is an "input", so to speak, it should be set

	// data demux
	void		ProcessAsfDataObject(HANDLE hOutPutFile, DWORD dwDataLen);
	void		WriteFrame(DWORD dwFrameNum, BOOL bIsKeyFrame, const char* pBuf, DWORD dwFrameLen);

// ------- per-stream data ------
	struct STREAM_PROPERTIES_OBJECT				m_StreamProps;
	struct EXTENDED_STREAM_PROPERTIES_OBJECT	m_ExtStrProp;
	struct PAYLOAD_EXTENSION_SYSTEMS			m_PayloadExtSysArray[MAX_PAYLOAD_EXTENSION_SYSTEMS];
	struct VC1_HEADER_SEQUENCE					m_AsfVc1Hdr;		// original, from ASF file
	struct VC1_HEADER_SEQUENCE					m_Vc1SeqAndEntryHdr;
	struct VC1_HEADER_SEQUENCE					m_Vc1EntryHdrOnly;

	DWORD	m_dwMetaParWidth;
	DWORD	m_dwMetaParHeight;

	// VC-1 related, when we do the demux. Prob should just make these local vars
	DWORD	m_dwFrameCounts[FRMTYPE_NUM_FRMTYPES];
	DWORD	m_dwTotalFrames;

public:

	inline BOOL	IsVideo() { return(m_bIsVideo); }
	inline BOOL	IsVC1() { return(m_bIsVc1); }
	inline DWORD GetAsfBitrate(void) { return(m_ExtStrProp.dwDataBitrate); }
	inline void SetInterlace(BOOL bIsInterlaced) { m_bIsInterlaced = bIsInterlaced; }
	inline void SetUseEntryHdrOnlyFlag(BOOL bUseEntryHdrOnly) { m_bUseEntryHdrOnly = bUseEntryHdrOnly; }

	~CAsfStream(void);
};
