/* *******************************************************************************************
The MIT License

Copyright (c) 2007 Steven G. Greenberg

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons
to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
******************************************************************************************* */

#include "stdafx.h"


#include "constants.h"
#include "GetBits.h"

#define GETBITBYIDX(pb,n) (((pb[n/8]) & (g_maskTbl[n%8])) ? 1 : 0)

const BYTE g_maskTbl[8] = {0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01};

CGetBits::CGetBits(const BYTE* pBuf, DWORD dwBufLen)
{
	m_pBuf = pBuf;
	m_dwBufLen = dwBufLen;
	m_dwRunningIdx = 0;
}

CGetBits::~CGetBits(void)
{
}

DWORD CGetBits::GetBits(DWORD dwNumBits)
{
	DWORD dwRtnVal;

	dwRtnVal = GetBitsByIndex(m_dwRunningIdx, dwNumBits);
	m_dwRunningIdx += dwNumBits;

	return(dwRtnVal);
}

BOOL CGetBits::ReSync32(DWORD dwStartCode)
{
	DWORD	dwByteIdx;
	BOOL	bRtnVal = FALSE;

	dwStartCode = NTOHL(dwStartCode);

	for (dwByteIdx = m_dwRunningIdx/8; m_dwBufLen > 5 && dwByteIdx < m_dwBufLen - 5; dwByteIdx++)
	{
		if (*((DWORD*)(&m_pBuf[dwByteIdx])) == dwStartCode)
		{
			m_dwRunningIdx = (dwByteIdx +4) * 8;
			bRtnVal = TRUE;
			break;
		}
	}
	return(bRtnVal);
}

DWORD CGetBits::GetBitsByIndex(DWORD dwStartBitIdx, DWORD dwNumBits)
{
	DWORD dwRtnVal = 0;

	if ((dwStartBitIdx + dwNumBits) / 8 < m_dwBufLen && dwNumBits <= 32)
	{
		for (DWORD dwIdx = dwStartBitIdx; dwIdx < dwStartBitIdx + dwNumBits; dwIdx++)
		{
			dwRtnVal *= 2;
			dwRtnVal |= GETBITBYIDX(m_pBuf, dwIdx);
		}
	}
	else
		_ASSERT_(0);

	return(dwRtnVal);
}
