/* *******************************************************************************************
The MIT License

Copyright (c) 2007 Steven G. Greenberg

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons
to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
******************************************************************************************* */

#include "stdafx.h"

#include <conio.h>
#include <stdarg.h>

#include "constants.h"
#include "log.h"

extern DWORD g_dwLogLevel;	// cheesy global

void DoLog(const char*  lpszFormat, va_list args);
void DoStdErrOut(const char*  lpszFormat, va_list args);

void Log(DWORD dwMsgLogType, const char* format, ...)
{
	va_list args;

	// Quick "verbosity" implementation by "level" only. This will probably get changed in the future.
	va_start(args,format);

	if (dwMsgLogType == LOG_TYPE_CONSOLE_ERROR)
		DoStdErrOut(format, args);
	
	else if (g_dwLogLevel >= dwMsgLogType)
		DoLog(format, args);
	
	va_end(args);

    return;
}

// print to stdout
void DoLog(const char* lpszFormat, va_list args)
{
	vprintf(lpszFormat, args);
}

// Print to "console" (stderr). Two versions - same difference I suppose, but I added
// the second one when I realized VC6 apparently doesn't have "_vcprintf" (the MS
// documentation notwithstanding)...

void DoStdErrOut(const char* lpszFormat, va_list args)
{
#if _MSC_VER >= 1400
	_vcprintf(lpszFormat, args);
#else
	vfprintf(stderr, lpszFormat, args);
#endif
}
