/* *******************************************************************************************
The MIT License

Copyright (c) 2007 Steven G. Greenberg

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons
to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
******************************************************************************************* */

#pragma once

#include "GetBits.h"
#include "PutBits.h"

// just keep the source table neat without making it wider ;)
#define SEQHEADER_TEXT		"\n---- Sequence Header ----\n"
#define ENTRYPOINT_TEXT		"\n---- Entry Point Header  ----\n"

const struct BIT_PARSE_TABLE InitSeqAndEntryTbl[] =
{
	{ _SEQUENCE_START_,      SEQHEADER_TEXT,          32, TRUE, SEQ_SC,_____, ___,  _,  __,                 }, // special entry
    { PROFILE,               "PROFILE",                2,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.1.1
    { LEVEL,                 "LEVEL",                  3,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.1.2
    { COLORDIFF_FORMAT,      "COLORDIFF_FORMAT",       2,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.1.3
    { FRMRTQ_POSTPROC,       "FRMRTQ_POSTPROC",        3,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.1.4.1
    { BITRTQ_POSTPROC,       "BITRTQ_POSTPROC",        5,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.1.4.2
    { POSTPROCFLAG,          "POSTPROCFLAG",           1,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.1.5
    { MAX_CODED_WIDTH,       "MAX_CODED_WIDTH",        12, FALSE,   0, _____, ___,  _,  __,                 }, // 6.1.6
    { MAX_CODED_HEIGHT,      "MAX_CODED_HEIGHT",       12, FALSE,   0, _____, ___,  _,  __,                 }, // 6.1.7
    { PULLDOWN,              "PULLDOWN",               1,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.1.8
    { INTERLACE,             "INTERLACE",              1,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.1.9
    { TFCNTRFLAG,            "TFCNTRFLAG",             1,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.1.10
    { FINTERPFLAG,           "FINTERPFLAG",            1,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.1.11
    { RESERVED,              "RESERVED",               1,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.1.12
    { PSF,                   "PSF",                    1,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.1.13
    { DISPLAY_EXT,           "DISPLAY_EXT",            1,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.1.14
    { _DISPLAY_EXT_COND_,    "",			           0,  FALSE,   0, DISPLAY_EXT, BNE, 1, HRD_PARAM_FLAG, }, // dummy entry
    { DISP_HORIZ_SIZE,       "-DISP_HORIZ_SIZE",       14, FALSE,   0, _____, ___,  _,  __,                 }, // 6.1.14.1
    { DISP_VERT_SIZE,        "-DISP_VERT_SIZE",        14, FALSE,   0, _____, ___,  _,  __,                 }, // 6.1.14.2
    { ASPECT_RATIO_FLAG,     "-ASPECT_RATIO_FLAG",     1,  FALSE,   0, _____, BNE,  1,  FRAMERATE_FLAG,     }, // 6.1.14.3
    { ASPECT_RATIO,          "--ASPECT_RATIO",         4,  FALSE,   0, _____, BNE,  15, FRAMERATE_FLAG,     }, // 6.1.14.3.1
    { ASPECT_HORIZ_SIZE,     "---ASPECT_HORIZ_SIZE",   8,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.1.14.3.2
    { ASPECT_VERT_SIZE,      "---ASPECT_VERT_SIZE",    8,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.1.14.3.3
    { FRAMERATE_FLAG,        "-FRAMERATE_FLAG",        1,  FALSE,   0, _____, BNE,  1,  COLOR_FORMAT_FLAG,  }, // 6.1.14.4
    { FRAMERATEIND,          "--FRAMERATEIND",         1,  FALSE,   0, _____, BNE,  0,  FRAMERATEEXP,       }, // 6.1.14.4.1
    { FRAMERATENR,           "--FRAMERATENR",          8,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.1.14.4.2
    { FRAMERATEDR,           "--FRAMERATEDR",          4,  FALSE,   0, _____, BRA,  0,  COLOR_FORMAT_FLAG,  }, // 6.1.14.4.3
    { FRAMERATEEXP,          "--FRAMERATEEXP",         16, FALSE,   0, _____, ___,  _,  __,                 }, // 6.1.14.4.4
    { COLOR_FORMAT_FLAG,     "-COLOR_FORMAT_FLAG",     1,  FALSE,   0, _____, BNE,  1,  HRD_PARAM_FLAG,     }, // 6.1.14.5
    { COLOR_PRIM,            "--COLOR_PRIM",           8,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.1.14.5.1
    { TRANSFER_CHAR,         "--TRANSFER_CHAR",        8,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.1.14.5.2
    { MATRIX_COEF,           "--MATRIX_COEF",          8,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.1.14.5.3
    { HRD_PARAM_FLAG,        "HRD_PARAM_FLAG",         1,  FALSE,   0, _____, BNE,  1,  _END_OF_SEQ_HDR_,   }, // 6.1.14.5.3
    { HRD_NUM_LEAKY_BUCKETS, "-HRD_NUM_LEAKY_BUCKETS", 5,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.1.15.1
    { BIT_RATE_EXPONENT,     "-BIT_RATE_EXPONENT",     4,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.1.15.1
    { BUFFER_SIZE_EXPONENT,  "-BUFFER_SIZE_EXPONENT",  4,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.1.15.1
	{ _LEAKY_BUCKETS_,       "",                       0,  FALSE,   0, _____, ___,  _,  __,                 }, // special entry
    { _END_OF_SEQ_HDR_,      "",                       0,  FALSE,   0, _____, ___,  _,  __,                 }, // special entry
    { _ENTRYPOINT_START_,    ENTRYPOINT_TEXT,         32, TRUE, ENT_SC,_____, ___,  _,  __,                 }, // special entry
    { BROKEN_LINK,           "BROKEN_LINK",            1,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.2.1
    { CLOSED_ENTRY,          "CLOSED_ENTRY",           1,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.2.2
    { PANSCAN_FLAG,          "PANSCAN_FLAG",           1,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.2.3
    { REFDIST_FLAG,          "REFDIST_FLAG",           1,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.2.4
    { LOOPFILTER,            "LOOPFILTER",             1,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.2.5
    { FASTUVMC,              "FASTUVMC",               1,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.2.6
    { EXTENDED_MV,           "EXTENDED_MV",            1,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.2.7
    { DQUANT,                "DQUANT",                 2,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.2.8
    { VSTRANSFORM,           "VSTRANSFORM",            1,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.2.9
    { OVERLAP,               "OVERLAP",                1,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.2.10
    { QUANTIZER,             "QUANTIZER",              2,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.2.11
	{ _HRD_FULLNESS_,        "",                       0,  FALSE,   0, _____, ___,  _,  __,                 }, // special entry
    { CODED_SIZE_FLAG,       "CODED_SIZE_FLAG",        1,  FALSE,   0, _____, BNE,  1,  _LABEL1_,            }, // 6.2.13
    { CODED_WIDTH,           "-CODED_WIDTH",           12, FALSE,   0, _____, ___,  _,  __,                 }, // 6.2.13.1
    { CODED_HEIGHT,          "-CODED_HEIGHT",          12, FALSE,   0, _____, ___,  _,  __,                 }, // 6.2.13.2
    { _LABEL1_,               "",                      0,  FALSE,   0, EXTENDED_MV, BNE, 1, RANGE_MAPY_FLAG,}, // special entry
    { EXTENDED_DMV,          "EXTENDED_DMV",           1,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.2.14
    { RANGE_MAPY_FLAG,       "RANGE_MAPY_FLAG",        1,  FALSE,   0, _____, BNE,  1,  RANGE_MAPUV_FLAG,   }, // 6.2.15
    { RANGE_MAPY,            "-RANGE_MAPY",            3,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.2.15.1
    { RANGE_MAPUV_FLAG,      "RANGE_MAPUV_FLAG",       1,  FALSE,   0, _____, BNE,  1,  _END_,              }, // 6.2.16
    { RANGE_MAPUV,           "-RANGE_MAPUV",           3,  FALSE,   0, _____, ___,  _,  __,                 }, // 6.2.16.1
    { _END_,                 "",                       0,  FALSE,   0, _____, ___,  _,  __,                 }, // dummy; separate from EOT for debug reasons
    { __EOT__,               "",                       0,  FALSE,   0, _____, ___,  _,  __,                 }, // end of table
 };

#define NUM_SE_TBL_ENTRIES	(sizeof(InitSeqAndEntryTbl) / sizeof(InitSeqAndEntryTbl[0]))


class CVc1Parse
{
public:
	CVc1Parse(void);
public:
	~CVc1Parse(void);


public:
	void SetFramePeriod(DWORD dwFrmPerMicroSecs);
	void SetPar(DWORD dwHoriz, DWORD dwVert);

	BOOL IsInterlaced(void);
	
	BOOL BitStringToTable(struct VC1_HEADER_SEQUENCE* pInputSeq);
	BOOL TableToBitString(struct VC1_HEADER_SEQUENCE* pOutputSeq, BOOL bSkipSeqStartCode);
	
	void PrintBitParseTable(void);

protected:

	struct BIT_PARSE_TABLE	m_SeqAndEntryTbl[NUM_SE_TBL_ENTRIES];
	struct LEAKY_BUCKET		m_LeakyBucketVals[MAX_LEAKY_BUCKET_VALS];
	BYTE					m_HrdFull[MAX_LEAKY_BUCKET_VALS];

	DWORD	m_dwNumLeakyBucketVals;

	DWORD	m_dwFrmRateNR;
	DWORD	m_dwFrmRateDR;
	DWORD	m_wFrmRateEXP;

	void CreateDefaultDisplayExtension(void);

	int		IndexOf(enum VC1_HDR_PARAMS nTargParam);
	void	SetTableValue(enum VC1_HDR_PARAMS nParm, DWORD dwVal);
	DWORD	GetTableValue(enum VC1_HDR_PARAMS nParm);
	
	void FillHrdFullnessTable(CGetBits* pBitsIn);
	void DumpHrdFullnessTable(CPutBits* pBitsOut);

	void FillLeakyBucketTable(CGetBits* pBitsIn);
	void DumpLeakyBucketTable(CPutBits* pBitsOut);
	
	void PrintLeakyBucketTable(void);
	void PrintHrdFullnessTable(void);
	void PrintMiscEntry(struct BIT_PARSE_TABLE* pEntry);
};
